@@ -13,6 +13,7 @@ static IMForwardEventStruct *current_forward_eve;
 
 static char callback_str_buffer[32];
 Window focus_win;
+static int timeout_handle;
 
 static void send_fake_key_eve()
 {
@@ -61,35 +62,35 @@ void send_text(char *text)
   char *utf8_gbtext = NULL;
 
   if (gb_output) {
-  u_int rn = 0 , wn = 0;
-  GError *err = NULL;
-  char *big5 = g_convert(text, len, "big5", "UTF-8", &rn, &wn, &err);
+    u_int rn = 0 , wn = 0;
+    GError *err = NULL;
+    char *big5 = g_convert(text, len, "big5", "UTF-8", &rn, &wn, &err);
 
-  if (err) {
-    dbg("utf8 -> gb  convert error %d %d\n", rn, wn);
-    return;
-  }
+    if (err) {
+      dbg("utf8 -> gb  convert error %d %d\n", rn, wn);
+      return;
+    }
 
-  err = NULL;
-  char *gbtext = g_convert(big5, wn, "GB2312", "big5", &rn, &wn, &err);
-  g_free(big5);
+    err = NULL;
+    char *gbtext = g_convert(big5, wn, "GB2312", "big5", &rn, &wn, &err);
+    g_free(big5);
 
-  if (err) {
-    dbg("big5 -> gb  convert error %d %d\n", rn, wn);
-    return;
-  }
+    if (err) {
+      dbg("big5 -> gb  convert error %d %d\n", rn, wn);
+      return;
+    }
 
-  err = NULL;
-  utf8_gbtext = g_convert(gbtext, wn, "UTF-8", "GB2312", &rn, &wn, &err);
-  g_free(gbtext);
+    err = NULL;
+    utf8_gbtext = g_convert(gbtext, wn, "UTF-8", "GB2312", &rn, &wn, &err);
+    g_free(gbtext);
 
-  if (err) {
-    dbg("gb -> utf8  convert error %d %d\n", rn, wn);
-    return;
-  }
+    if (err) {
+      dbg("gb -> utf8  convert error %d %d\n", rn, wn);
+      return;
+    }
 
-  text = utf8_gbtext;
-  len = wn;
+    text = utf8_gbtext;
+    len = wn;
   }
 
   int requiredN = len + 1 + output_bufferN;
@@ -187,6 +188,11 @@ void hide_in_win(ClientState *cs)
   dbg("hide_in_win %d\n", ic->in_method);
 #endif
 
+  if (timeout_handle) {
+    g_source_remove(timeout_handle);
+    timeout_handle = 0;
+  }
+
   switch (cs->in_method) {
     case 3:
       hide_win_pho();
@@ -237,6 +243,7 @@ void show_in_win(ClientState *cs)
   show_win_stautus();
 }
 
+
 void move_win_gtab(int x, int y);
 void move_win_int(int x, int y);
 void move_win0(int x, int y);
@@ -614,6 +621,14 @@ int feedkey_intcode(KeySym key);
 void tsin_set_eng_ch(int nmod);
 static KeySym last_keysym;
 
+gboolean timeout_raise_window()
+{
+  timeout_handle = 0;
+  show_in_win(current_CS);
+  return FALSE;
+}
+
+
 // return TRUE if the key press is processed
 gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
 {
@@ -677,6 +692,12 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
       return TRUE;
     }
 
+    if (keysym == ',') {
+      create_win_sym();
+
+      return TRUE;
+    }
+
     int kidx = gcin_switch_keys_lookup(keysym);
     if (kidx < 0)
       return FALSE;
@@ -701,6 +722,9 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
      return TRUE;
   }
 
+  if (current_CS->b_raise_window && !timeout_handle) {
+    timeout_handle = g_timeout_add(200, timeout_raise_window, NULL);
+  }
 
   switch(current_CS->in_method) {
     case 3:
