@@ -960,18 +960,6 @@ gboolean feedkey_gtab(KeySym key, int kbstate)
 
   if (same_pho_query_state == SAME_PHO_QUERY_pho_select)
     return feedkey_pho(key, 0);
-#if 0
-  if (current_CS->b_half_full_char) {
-     char *s = half_char_to_full_char(key);
-     if (!s)
-       return 0;
-     char tt[CH_SZ+1];
-
-     memcpy(tt, s, CH_SZ); tt[CH_SZ]=0;
-     send_text(tt);
-     return 1;
-  }
-#endif
 
 
   if (key < 127 && cur_inmd->keymap[key]) {
@@ -981,8 +969,12 @@ gboolean feedkey_gtab(KeySym key, int kbstate)
      char ucase = toupper(key);
      if (cur_inmd->keymap[lcase] != cur_inmd->keymap[ucase])
        goto next;
+
+     if (gtab_capslock_in_eng && (kbstate&LockMask))
+       return 0;
   }
 
+
 shift_proc:
   if ((kbstate & ShiftMask) && key!='*' && (key!='?' || gtab_shift_phrase_key && !ci)) {
     if (gtab_shift_phrase_key)
@@ -1048,8 +1040,10 @@ shift_proc:
       if (ci) {
         reset_gtab_all();
         return 1;
-      } else
+      } else {
+        ClrIn();
         return 0;
+      }
     case '<':
       if (wild_mode) {
         if (wild_page >= cur_inmd->M_DUP_SEL) wild_page-=cur_inmd->M_DUP_SEL;
@@ -1094,6 +1088,9 @@ shift_proc:
           sel1st_i = 0;
         }
 
+        if (_gtab_space_auto_first == GTAB_space_auto_first_nofull && exa_match > 1)
+          bell();
+
         if (seltab[sel1st_i][0]) {
 //          dbg("last_full %d %d\n", last_full,spc_pressed);
           if (gtab_full_space_auto_first || spc_pressed) {
@@ -1151,14 +1148,13 @@ shift_proc:
       }
     default:
 next:
+
       if (invalid_spc) {
         ClrIn();
       }
       if (key>=XK_KP_0 && key<=XK_KP_9) {
-#if 0
-        if (!spc_pressed)
-          return 0;
-#endif
+        if (!ci)
+          return FALSE;
         if (!strncmp(cur_inmd->filename, "dayi", 4)) {
           key = key - XK_KP_0 + '0';
           is_keypad = TRUE;
@@ -1490,14 +1486,17 @@ next_pg:
       goto refill;
     } else
     if (!more_pg) {
-      if (gtab_dup_select_bell && (gtab_full_space_auto_first || spc_pressed || last_full))
-        bell();
+      if (gtab_dup_select_bell && (gtab_disp_partial_match || gtab_pre_select)) {
+        if (spc_pressed || gtab_full_space_auto_first || last_full && gtab_press_full_auto_send)
+          bell();
+      }
     }
   }
 
 Disp_opt:
-  if (gtab_disp_partial_match || gtab_pre_select || exa_match > 1)
-  disp_selection(phrase_selected);
+  if (gtab_disp_partial_match || gtab_pre_select || (exa_match > 1 && (spc_pressed || gtab_press_full_auto_send)) ) {
+       disp_selection(phrase_selected);
+  }
 
   return 1;
 }
