@@ -6,7 +6,8 @@ typedef enum {
   GCIN_req_key_release = 2,
   GCIN_req_focus_in = 4,
   GCIN_req_focus_out = 8,
-  GCIN_req_set_cursor_location = 16,
+  GCIN_req_set_cursor_location = 0x10,
+  GCIN_req_set_flags = 0x20
 } GCIN_req_t;
 
 enum {
